///////////////////////////////////////////////////////////////////////////////
//
//            (C) Copyright 2009-2010 OSR Open Systems Resources, Inc.
//            All Rights Reserved
//
//    This work is distributed under the OSR Non-Commercial Software License which is provided
//    at "http://www.osronline.com/page.cfm?name=NonCommLicense" in the hope that it will be
//    enlightening, but WITHOUT ANY WARRANTY; without even the implied warranty of MECHANTABILITY
//    or FITNESS FOR A PARTICULAR PURPOSE.
//
//            OSR Open Systems Resources, Inc.
//            105 Route 101A Suite 19
//            Amherst, NH 03031  (603) 595-6500 FAX: (603) 595-6503
//
//
//    MODULE:
//
//        $File: //depot/tools/UMERESOURCES/inc/UmEResource.h $
//
//    ABSTRACT:
//
//     This module defines Executive Resources to be used in user mode.
//
//    AUTHOR:
//
//        OSR Open Systems Resources, Inc.
// 
//    REVISION:   
//
//        $Revision: #1 $
//
//
///////////////////////////////////////////////////////////////////////////////
#ifndef __UMERESOURCE_H__
#define __UMERESOURCE_H__

//
// The Win32 call GetCurrentThreadId() is used to retrieve the thread id.
//
#define ERESOURCE_THREAD DWORD

//
// OSRRESOURCE structure used by code.
//
typedef struct _ERESOURCE 
{
	PVOID	eResource;
} OSRRESOURCE,*POSRRESOURCE;

typedef OSRRESOURCE OSRRESOURCE;
typedef POSRRESOURCE POSRRESOURCE;

//
// EResource routines.   This routines are intended to behave exactly like
// the kernel mode routines behave, except that they operate at IRQL 
// PASSIVE LEVEL.
//
// So, recursive acquisition is allowed, as long as the acquisition rules are
// followed, i.e if you have it shared, you can't acquire it exclusive, but if you 
// have it exclusive and try to get it shared, your shared acquisition becomes a 
// exclusive acquisition.
//
//
// Note:  If you do something illegal with the locks, they will throw an exception.
// An exception indicates an illegal lock operation has taken place and cannot
// be recovered from.   The exception code 0xC000000X indicates where the exception
// took place.
//
// Exception codes:
//
//  0xC0000001 - IsResourceAcquiredShared - invalid lock state, corruption?
//  0xC0000002 - ReleaseResource - resource not owned by caller
//  0xC0000003 - ConvertExclusiveToShared - resource not owned exclusive
//  0xC0000004 - [NOT USED] ConvertExclusiveToShared - resource owned more than once
//  0xC0000005 - ConvertExclusiveToShared - resource not owned by caller.
//  0xC0000006 - ReleaseResourceForThread - resource not owned by input thread
//  0xC0000007 - DeleteResource - resource still in use.
//	0xC0000008 - AcquireResourceExclusive - already have the lock shared
//  0xC0000009 - Generic Error - Resource not initialized.
//  0xC000000A - OSRInitializeResourceLite - Resource already initialized.
//
extern "C" {

BOOLEAN
__cdecl OSRAcquireResourceExclusiveLite (
    __inout POSRRESOURCE Resource,
    __in BOOLEAN Wait
    );

BOOLEAN
__cdecl OSRAcquireResourceSharedLite (
    __inout POSRRESOURCE Resource,
    __in BOOLEAN Wait
    );

BOOLEAN
__cdecl OSRAcquireSharedStarveExclusive(
    __inout POSRRESOURCE Resource,
    __in BOOLEAN Wait
    );

BOOLEAN
__cdecl OSRIsResourceAcquiredExclusiveLite (
    __in POSRRESOURCE Resource
    );

BOOLEAN
__cdecl OSRIsResourceAcquiredSharedLite (
    __in POSRRESOURCE Resource
    );

VOID
__cdecl OSRReleaseResourceLite(
    __inout POSRRESOURCE Resource
    );

ERESOURCE_THREAD 
__cdecl OSRGetCurrentResourceThread( 
    VOID
    );

VOID
__cdecl OSRConvertExclusiveToSharedLite(
    __inout POSRRESOURCE Resource
    );

VOID
__cdecl OSRReleaseResourceForThreadLite(
    __inout POSRRESOURCE Resource,
    __in ERESOURCE_THREAD ResourceThreadId
    );

NTSTATUS
__cdecl OSRInitializeResourceLite (
    __out POSRRESOURCE Resource
    );

NTSTATUS
__cdecl OSRDeleteResourceLite (
    __inout POSRRESOURCE Resource
    );

ULONG
__cdecl OSRGetSharedWaiterCount(
    __inout POSRRESOURCE Resource
	);

ULONG
__cdecl OSRGetExclusiveWaiterCount(
    __inout POSRRESOURCE Resource
	);

};

#endif __UMERESOURCE_H__